import argparse

from lusidfeature.feature_extractor import extract_all_features_from_package
from lusidfeature.feature_file_writer import write_features_to_file


def extract_features_to_file(argv):
    ap = argparse.ArgumentParser(description="Get arguments from command line")
    ap.add_argument('-p', '--package', required=True, help='package name from the root folder sdk, not including the '
                                                           'root folder. Eg if root folder is "sdk", then package name '
                                                           'should be "tests.tutorials", not "sdk.tests.tutorials", '
                                                           'and also not "tutorials"')
    ap.add_argument('-o', '--outpath', required=True, help='Path and name of the features file to be created. Eg. '
                                                           '<path>/features.txt')
    ap.add_argument('-r', '--root', required=True, help='The path of root directory from which the decorator '
                                                                  'scanner should start traversing packages and '
                                                                  'modules. Recommended to be the root SDK folder or '
                                                                  'src folder rather than the base project path.')

    args = vars(ap.parse_args())
    package_name = args["package"]
    filepath = args["outpath"]
    root_dir = args["root"]

    feature_list = extract_all_features_from_package(package_name, root_dir)
    write_features_to_file(feature_list, filepath)
    print("File written to: " + filepath)



import os


def write_features_to_file(feature_list, filepath):
    text_to_write = "\n".join(feature_list)
    with open(filepath, "w+") as f:
        f.write(text_to_write)


def remove_file(filepath):
    is_file = os.path.isfile(filepath)
    if is_file:
        os.remove(filepath)


def read_file(filepath):
    with open(filepath, "r") as f:
        features_from_file = f.read().splitlines()
    return features_from_file



def lusid_feature(*feature_codes):
    if len(feature_codes) < 1:
        raise ValueError("lusid_feature error: Decorator requires at least some input.")

    for feature_code in feature_codes:
        if type(feature_code) != str:
            raise ValueError("lusid_feature error: Decorator requires a string input parameter.")

        if feature_code == "":
            raise ValueError("lusid_feature error: Some decorated methods have no value passed. "
                             "Please make sure each lusid_feature decorator has a value code passed.")

    def wrap_method(decorated_method):
        def get_decorated_method(*decorated_method_args):
            return decorated_method
        get_decorated_method.decorator_value = list(feature_codes)  # <-- store the feature
        return get_decorated_method

    return wrap_method






import tempfile
import os


class FeaturesTempFileManager:
    """
    This class is used for managing the creation and deletion of temporary feature files
    """
    @staticmethod
    def create_temp_file(data):
        """
        Creates a temporary file from extracted lusidfeature which is written to the file as a text file
        :param str data: The data is passed as a text string and saved to the temp file
        :return: tempfile.NamedTemporaryFile features_file: The temporary file
        """
        features_file = tempfile.NamedTemporaryFile(mode="w+", delete=False)
        features_file.write(data)
        features_file.seek(0)
        return features_file

    @staticmethod
    def delete_temp_file(file):
        """
        Deletes a temporary file
        :param tempfile.NamedTemporaryFile file: The file to delete
        :return: None
        """
        file.close()
        os.unlink(file.name)



import importlib
import inspect
import pkgutil

from lusidfeature.get_project_root import get_project_root


def validate_feature_list(feature_list):
    unique_features = set(feature_list)
    if len(unique_features) != len(feature_list):
        sorted_features = sorted(feature_list)
        for i, code in enumerate(sorted_features):
            if sorted_features[i] == sorted_features[i + 1]:
                raise ValueError(f'lusid_feature error: Feature code "{code}" is a duplicate. '
                                 'Please make sure each feature code is unique. Also make sure lusid_feature '
                                 'decorator is on top of any other decorators for that function/method.')


def get_decorator_values_from_classes(module):
    classes = inspect.getmembers(module, predicate=inspect.isclass)
    for cls_name, cls in classes:
        # get all class methods that are not inherited. Avoids duplicate extraction
        methods = [(n, m) for (n, m) in cls.__dict__.items() if callable(m)]
        for method_name, method in methods:
            if hasattr(method, "decorator_value"):
                yield method.decorator_value


def get_decorator_values_from_functions(module):
    functions = inspect.getmembers(module, predicate=inspect.isfunction)
    for function_name, function in functions:
        if hasattr(function, "decorator_value"):
            yield function.decorator_value


def extract_all_features_from_package(package_name, root):
    feature_list = []
    for importer, name, is_pkg in pkgutil.walk_packages([root]):
        if is_pkg or not name.startswith(package_name):
            continue

        module = importlib.import_module(name)

        for feature_codes in get_decorator_values_from_classes(module):
            feature_list = feature_list + feature_codes

        for feature_codes in get_decorator_values_from_functions(module):
            feature_list = feature_list + feature_codes

    validate_feature_list(feature_list)
    return feature_list



from setuptools import setup


def readme():
    with open('README.md') as f:
        return f.read()


setup(
    name='lusidfeature',
    version='0.2.1',
    description='This package will allow to run the main file and retrieve a list of decorated feature methods in a cls',
    long_description=readme(),
    long_description_content_type='text/markdown',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    url='https://github.com/finbourne/lusid-features-python.git',
    author='FINBOURNE Technology',
    author_email='engineering@finbourne.com',
    license='MIT',
    keywords=["OpenAPI", "FINBOURNE", "LUSID", "LUSID API"],
    packages=['lusidfeature'],
    install_requires=['parameterized>=0.7.4'],
    include_package_data=True,
    zip_safe=False,
    python_requires=">=3.6"
)
